import os
import argparse
import shutil
import re

from colorama import init, Fore, Style

init(autoreset=True)

ESTRUTURA = {
    "src": ["bots", "core", "integracoes", "utils", "api"],
    "config": [],
    "dados": [],
    "logs": [],
    "tests": [],
}

ARQUIVOS_BASE = [
    "requirements.txt",
    "README.md",
    "main.py"
]

def validar_nome_projeto(nome):
    """Verifica se o nome do projeto é válido (sem espaços e caracteres especiais)."""
    if re.match(r'^[a-zA-Z0-9_-]+$', nome):
        return True
    print("Erro: O nome do projeto contém caracteres inválidos! Use apenas letras, números, hífens ou underscores.")
    return False

def criar_arquitetura(caminho_base):
    """Cria a estrutura do projeto no caminho especificado."""
    if not os.path.exists(caminho_base):
        os.makedirs(caminho_base)

    for pasta, subpastas in ESTRUTURA.items():
        pasta_path = os.path.join(caminho_base, pasta)
        os.makedirs(pasta_path, exist_ok=True)
        for subpasta in subpastas:
            os.makedirs(os.path.join(pasta_path, subpasta), exist_ok=True)

    for arquivo in ARQUIVOS_BASE:
        caminho_arquivo = os.path.join(caminho_base, arquivo)
        if not os.path.exists(caminho_arquivo):  # Evita sobrescrever arquivos já existentes
            with open(caminho_arquivo, "w", encoding="utf-8") as f:
                if arquivo == "README.md":
                    f.write(f"# Projeto\n\nDescrição do projeto.")
                elif arquivo == "requirements.txt":
                    f.write("# Lista de dependências do projeto\n")
                elif arquivo == "main.py":
                    f.write("# Ponto de entrada do projeto\n\nif __name__ == '__main__':\n    print('Projeto iniciado!')")

    inicializar_git(caminho_base)
    print(Fore.GREEN + "✅ Estrutura do projeto foi gerada com sucesso! 🎉")
    print(Fore.YELLOW + f"🔧 Estrutura do projeto foi configurada corretamente com os seguintes diretórios e arquivos:")
    print(Fore.CYAN + f"    ├── /src/                # Código-fonte principal")
    print(Fore.CYAN + f"    ├── /config/             # Arquivos de configuração")
    print(Fore.CYAN + f"    ├── /dados/              # Entrada/Saída de arquivos")
    print(Fore.CYAN + f"    ├── /logs/               # Logs detalhados de execução")
    print(Fore.CYAN + f"    ├── /tests/              # Testes unitários")
    print(Fore.CYAN + f"    ├── requirements.txt     # Dependências do projeto")
    print(Fore.CYAN + f"    ├── README.md            # Documentação do projeto")
    print(Fore.CYAN + f"    └── main.py              # Ponto de entrada do projeto\n")
    print(Fore.GREEN + f"🚀 Agora você pode começar a desenvolver seu Robo 🤖!\n")

def inicializar_git(caminho_base):
    """Inicializa um repositório Git no diretório do projeto e cria um .gitignore padrão."""
    gitignore_path = os.path.join(caminho_base, ".gitignore")
    if not os.path.exists(gitignore_path):  # Evita sobrescrever .gitignore existente
        os.system(f"cd {caminho_base} && git init")
        with open(gitignore_path, "w") as f:
            f.write("""
# Python
__pycache__/
*.pyc
*.pyo
*.pyd
venv/
.env
logs/
""")

def main():
    parser = argparse.ArgumentParser(description="Ferramenta para criar estrutura de projetos RPA.")
    parser.add_argument("comando", choices=["start", "create"], help="Comando para iniciar o projeto")
    parser.add_argument("nome", nargs="?", help="Nome do projeto (necessário para 'create')")
    args = parser.parse_args()
    
    if args.comando == "start":
        criar_arquitetura(os.getcwd())  # Gera a estrutura na pasta atual
    elif args.comando == "create":
        if args.nome:
            caminho_base = os.path.join(os.getcwd(), args.nome)
            if os.path.exists(caminho_base):
                print(Fore.RED + f"Erro: O diretório '{args.nome}' já existe!")
            else:
                criar_arquitetura(caminho_base)
        else:
            print(Fore.RED + "Erro: O comando 'create' requer um nome de projeto!")

if __name__ == "__main__":
    main()